package ch.psi.daq.queryrest.config;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.util.StringUtils;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Sets;

import ch.psi.daq.common.statistic.StorelessStatistics;
import ch.psi.daq.domain.cassandra.ChannelEvent;
import ch.psi.daq.query.config.QueryClientConfig;
import ch.psi.daq.query.config.QueryConfig;
import ch.psi.daq.queryrest.model.PropertyFilterMixin;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;

@Configuration
@PropertySource(value = {"classpath:rest.properties"})
@PropertySource(value = {"file:${user.home}/.config/daq/rest.properties"}, ignoreResourceNotFound = true)
public class QueryRestConfig {

   @Autowired
   private Environment env;

   // a nested configuration
   // this guarantees that the ordering of the properties file is as expected
   // see:
   // https://jira.spring.io/browse/SPR-10409?focusedCommentId=101393&page=com.atlassian.jira.plugin.system.issuetabpanels:comment-tabpanel#comment-101393
   @Configuration
   @Import({QueryConfig.class, QueryClientConfig.class})
   static class InnerConfiguration {
   }

   @Bean
   public JsonFactory jsonFactory() {
      return new JsonFactory();
   }

   @Bean
   public ResponseStreamWriter responseStreamWriter() {
      return new ResponseStreamWriter();
   }

   @Bean
   public ObjectMapper objectMapper() {
      ObjectMapper mapper = new ObjectMapper();
      // only include non-null values
      mapper.setSerializationInclusion(Include.NON_NULL);
      // Mixin which is used dynamically to filter out which properties get serialised and which
      // won't. This way, the user can specify which columns are to be received.
      mapper.addMixIn(ChannelEvent.class, PropertyFilterMixin.class);
      mapper.addMixIn(StorelessStatistics.class, PropertyFilterMixin.class);
      return mapper;
   }
   
   @Bean
   private Set<String> defaultResponseFields() {
      List<String> defaultFields = Arrays.asList(
            StringUtils.commaDelimitedListToStringArray(
                  env.getProperty("rest.default.response.fields")
            ));
      Set<String> defaultResponseFields = Sets.newHashSet(defaultFields.iterator());
      return defaultResponseFields;
   }
}
